<?php

class Product extends ProductCore
{	
	
	/** @var int Number of select fields */
	public $select_fields;
	
	const CUSTOMIZE_SELECTFIELD = 2;
	
	public function __construct($id_product = null, $full = false, $id_lang = null, $id_shop = null, Context $context = null)
	{ 
		//'select_fields' => array('type' => self::TYPE_INT, 'shop' => true, 'validate' => 'isUnsignedInt'),
		self::$definition['fields']['select_fields'] = array('type' => self::TYPE_INT,'shop' => true, 'validate' => 'isUnsignedInt');
		parent::__construct($id_product,$full,$id_lang,$id_shop,$context);
	}
	
	
	public function createLabelsAndCombos($uploadable_files, $text_fields,$select_fields){
		$languages = Language::getLanguages();
		if ((int)$uploadable_files > 0)
			for ($i = 0; $i < (int)$uploadable_files; $i++)
				if (!$this->_createLabel($languages, Product::CUSTOMIZE_FILE))
					return false;
	
		if ((int)$text_fields > 0)
			for ($i = 0; $i < (int)$text_fields; $i++)
				if (!$this->_createLabel($languages, Product::CUSTOMIZE_TEXTFIELD))
					return false;
				
		if ((int)$select_fields > 0)
			for ($i = 0; $i < (int)$select_fields; $i++)
				if (!$this->_createLabelAndCombo($languages))
					return false;
		
		return true;
	}
	
	
	protected function _createLabelAndCombo(&$languages)
	{			
		// Label insertion
		if (!Db::getInstance()->execute('
			INSERT INTO `'._DB_PREFIX_.'customization_field` (`id_product`, `type`, `required`)
			VALUES ('.(int)$this->id.', '.(int)Product::CUSTOMIZE_SELECTFIELD.', 0)') ||
				!$id_customization_field = (int)Db::getInstance()->Insert_ID())
			return false;
	
		// Multilingual label name creation
		$values = '';
		foreach ($languages as $language)
			$values .= '('.(int)$id_customization_field.', '.(int)$language['id_lang'].', \'\'), ';
	
		$values = rtrim($values, ', ');
		if (!Db::getInstance()->execute('
			INSERT INTO `'._DB_PREFIX_.'customization_field_lang` (`id_customization_field`, `id_lang`, `name`)
			VALUES '.$values))
			return false;
		
		
		//Multilingual Combo creation
		if (!Db::getInstance()->execute('
			INSERT INTO `'._DB_PREFIX_.'customization_select_field_lang` (`id_customization_field`, `id_lang`, `combo`)
			VALUES '.$values))
			return false;
		
		// Set cache of feature detachable to true
		Configuration::updateGlobalValue('PS_CUSTOMIZATION_FEATURE_ACTIVE', '1');
	
		return true;
	}
	
	public function updateLabelsAndCombos()
	{
		$has_required_fields = 0;
		foreach ($_POST as $field => $value){
			/* Label update */
			if (strncmp($field, 'label_', 6) == 0)
			{
				if (!$tmp = $this->_checkLabelField($field, $value))
					return false;
				/* Multilingual label name update */
				if (!Db::getInstance()->execute('
						INSERT INTO `'._DB_PREFIX_.'customization_field_lang`
						(`id_customization_field`, `id_lang`, `name`) VALUES ('.(int)$tmp[2].', '.(int)$tmp[3].', \''.pSQL($value).'\')
						ON DUPLICATE KEY UPDATE `name` = \''.pSQL($value).'\''))
					return false;
				$is_required = isset($_POST['require_'.(int)$tmp[1].'_'.(int)$tmp[2]]) ? 1 : 0;
				$has_required_fields |= $is_required;
				/* Require option update */
				if (!Db::getInstance()->execute(
						'UPDATE `'._DB_PREFIX_.'customization_field`
						SET `required` = '.(int)$is_required.'
						WHERE `id_customization_field` = '.(int)$tmp[2]))
					return false;
			}
		
			/* Combo update */
			if (strncmp($field, 'combo_', 6) == 0)
			{
				if (!$tmp = $this->_checkLabelField($field, $value))
					return false;
				/* Multilingual combo  update */
				$value=trim($value);
				if (!Db::getInstance()->execute('
						INSERT INTO `'._DB_PREFIX_.'customization_select_field_lang`
						(`id_customization_field`, `id_lang`, `combo`) VALUES ('.(int)$tmp[2].', '.(int)$tmp[3].', \''.pSQL($value).'\')
						ON DUPLICATE KEY UPDATE `combo` = \''.pSQL($value).'\''))
					return false;
			}
		}
		
		if ($has_required_fields && !ObjectModel::updateMultishopTable('product', array('customizable' => 2), 'a.id_product = '.(int)$this->id))
			return false;
	
		if (!$this->_deleteOldLabelsAndCombos())
			return false;
	
		return true;
	}
	
	protected function _deleteOldLabelsAndCombos()
	{
		$max = array(
				Product::CUSTOMIZE_FILE => (int)Tools::getValue('uploadable_files'),
				Product::CUSTOMIZE_TEXTFIELD => (int)Tools::getValue('text_fields'),
				Product::CUSTOMIZE_SELECTFIELD => (int)Tools::getValue('select_fields')
		);
	
		/* Get customization field ids */
		if (($result = Db::getInstance()->executeS(
				'SELECT `id_customization_field`, `type`
			FROM `'._DB_PREFIX_.'customization_field`
			WHERE `id_product` = '.(int)$this->id.'
			ORDER BY `id_customization_field`')
		) === false)
			return false;
	
		if (empty($result))
			return true;
	
		$customization_fields = array(
				Product::CUSTOMIZE_FILE => array(),
				Product::CUSTOMIZE_TEXTFIELD => array(),
				Product::CUSTOMIZE_SELECTFIELD => array()
		);
	
		foreach ($result as $row)
			$customization_fields[(int)$row['type']][] = (int)$row['id_customization_field'];
	
		$extra_file = count($customization_fields[Product::CUSTOMIZE_FILE]) - $max[Product::CUSTOMIZE_FILE];
		$extra_text = count($customization_fields[Product::CUSTOMIZE_TEXTFIELD]) - $max[Product::CUSTOMIZE_TEXTFIELD];
		$extra_select = count($customization_fields[Product::CUSTOMIZE_SELECTFIELD]) - $max[Product::CUSTOMIZE_SELECTFIELD];
	
		/* If too much inside the database, deletion */
		if ($extra_file > 0 && count($customization_fields[Product::CUSTOMIZE_FILE]) - $extra_file >= 0 &&
		(!Db::getInstance()->execute(
				'DELETE FROM `'._DB_PREFIX_.'customization_field`
			WHERE `id_product` = '.(int)$this->id.'
			AND `type` = '.Product::CUSTOMIZE_FILE.'
			AND `id_customization_field` >= '.(int)$customization_fields[Product::CUSTOMIZE_FILE][count($customization_fields[Product::CUSTOMIZE_FILE]) - $extra_file]
		)
				|| !Db::getInstance()->execute(
						'DELETE FROM `'._DB_PREFIX_.'customization_field_lang`
			WHERE `id_customization_field`
			NOT IN (
				SELECT `id_customization_field`
				FROM `'._DB_PREFIX_.'customization_field`
			)'
				)))
			return false;
	
		if ($extra_text > 0 && count($customization_fields[Product::CUSTOMIZE_TEXTFIELD]) - $extra_text >= 0 &&
		(!Db::getInstance()->execute(
				'DELETE FROM `'._DB_PREFIX_.'customization_field`
			WHERE `id_product` = '.(int)$this->id.'
			AND `type` = '.Product::CUSTOMIZE_TEXTFIELD.'
			AND `id_customization_field` >= '.(int)$customization_fields[Product::CUSTOMIZE_TEXTFIELD][count($customization_fields[Product::CUSTOMIZE_TEXTFIELD]) - $extra_text]
		)
				|| !Db::getInstance()->execute(
						'DELETE FROM `'._DB_PREFIX_.'customization_field_lang`
			WHERE `id_customization_field`
			NOT IN (
				SELECT `id_customization_field`
				FROM `'._DB_PREFIX_.'customization_field`
			)'
				)))
			return false;
		
		if ($extra_select > 0 && count($customization_fields[Product::CUSTOMIZE_SELECTFIELD]) - $extra_select >= 0 &&
		(!Db::getInstance()->execute(
				'DELETE FROM `'._DB_PREFIX_.'customization_field`
			WHERE `id_product` = '.(int)$this->id.'
			AND `type` = '.Product::CUSTOMIZE_SELECTFIELD.'
			AND `id_customization_field` >= '.(int)$customization_fields[Product::CUSTOMIZE_SELECTFIELD][count($customization_fields[Product::CUSTOMIZE_SELECTFIELD]) - $extra_select]
		)
				|| !Db::getInstance()->execute(
						'DELETE FROM `'._DB_PREFIX_.'customization_field_lang`
			WHERE `id_customization_field`
			NOT IN (
				SELECT `id_customization_field`
				FROM `'._DB_PREFIX_.'customization_field`
			)'
				)/*Ahora toca además borrar los combos*/
				|| !Db::getInstance()->execute(
						'DELETE FROM `'._DB_PREFIX_.'customization_select_field_lang`
			WHERE `id_customization_field`
			NOT IN (
				SELECT `id_customization_field`
				FROM `'._DB_PREFIX_.'customization_field`
			)'
				)
				
				))
			return false;
	
		// Refresh cache of feature detachable
		Configuration::updateGlobalValue('PS_CUSTOMIZATION_FEATURE_ACTIVE', Customization::isCurrentlyUsed());
	
		return true;
	}
	
	
	/*
	 * Añadimos el combo a la información de los campos personalizables aunque sólo estará relleno en el caso de type==2 (SELECTFIELD)
	 */
	public function getCustomizationFields($id_lang = false)
	{
		if (!Customization::isFeatureActive())
			return false;
	
		if (!$result = Db::getInstance()->executeS('
			SELECT cf.`id_customization_field`, cf.`type`, cf.`required`, cfl.`name`, cfl.`id_lang`,csfl.`combo`
			FROM `'._DB_PREFIX_.'customization_field` cf
			NATURAL JOIN `'._DB_PREFIX_.'customization_field_lang` cfl 
			LEFT JOIN `'._DB_PREFIX_.'customization_select_field_lang` csfl ON (cf.id_customization_field=csfl.id_customization_field AND csfl.id_lang=cfl.id_lang)
			WHERE cf.`id_product` = '.(int)$this->id.($id_lang ? ' AND cfl.`id_lang` = '.(int)$id_lang : '').'
			ORDER BY cf.`id_customization_field`'))
			return false;
	
		if ($id_lang)
			return $result;
	
		$customization_fields = array();
		foreach ($result as $row)
			$customization_fields[(int)$row['type']][(int)$row['id_customization_field']][(int)$row['id_lang']] = $row;
	
		return $customization_fields;
	}
	
}

